package com.book.push.config;

import com.book.push.action.PushAction;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import redis.clients.jedis.JedisPubSub;

/**
 * created in 2021/9/7
 * Project: book-store
 *
 * @author win7
 */
@Configuration
@Slf4j
public class KeyExpiredListener extends JedisPubSub {
    public static final String KEY_PREFIX_MP_SUBSCRIBE = "mp_subscribe";
    public static final String KEY_PREFIX_MP_SIGN = "mp_sign";
    public static final String KEY_PREFIX_BOOK_CONTINUE_READ = "book_read";
    public static final String KEY_PREFIX_ORDER_NOPAY = "order_nopay";
    public static final String KEY_PREFIX_MP_NOSIGN = "mp_nosign";
    public static final String prefix = "auto_push:";
    @Autowired
    private PushAction pushAction;

    @Bean
    public KeyExpiredListener getBean() {
        return new KeyExpiredListener();
    }

    @Override
    public void onPSubscribe(String pattern, int subscribedChannels) {
        System.out.println("onPSubscribe " + pattern + "  " + subscribedChannels);
    }

    @Override
    public void onPMessage(String pattern, String channel, String message) {
        log.info("过期key：" + message);
        //收到消息key的键值
        if (StringUtils.isEmpty(message)) {
            return;
        }
        if (!message.startsWith(prefix)) {
            return;
        }


        String[] split = message.split(":");
        String actionString = split[1];
        Integer platformId = Integer.valueOf(split[2]);
        String appid = split[3];
        String openid = split[4];
        log.info("智能推送:platformId:{},appid:{},openid:{}", platformId, appid, openid);
        if (actionString.startsWith(KEY_PREFIX_MP_SUBSCRIBE)) {
            pushAction.pushBySubscribe(platformId, appid, openid);

        } else if (actionString.startsWith(KEY_PREFIX_MP_SIGN)) {


        } else if (actionString.startsWith(KEY_PREFIX_BOOK_CONTINUE_READ)) {


        } else if (actionString.startsWith(KEY_PREFIX_ORDER_NOPAY)) {


        } else if (actionString.startsWith(KEY_PREFIX_MP_NOSIGN)) {


        }
    }

    protected String getKey(Integer platformId, String appid, String openid, String keyPrefix) {
        return prefix + keyPrefix + ":" + platformId + ":" + appid + ":" + openid;
    }


    public String getKeyMpSubscribe(Integer platformId, String appid, String openid) {
        return getKey(platformId, appid, openid, KEY_PREFIX_MP_SUBSCRIBE);
    }

    public String getKeyMpSign(Integer platformId, String appid, String openid) {
        return getKey(platformId, appid, openid, KEY_PREFIX_MP_SIGN);
    }

    public String getKeyPrefixBookContinueRead(Integer platformId, String appid, String openid) {
        return getKey(platformId, appid, openid, KEY_PREFIX_BOOK_CONTINUE_READ);
    }

    public String getKeyPrefixOrderNopay(Integer platformId, String appid, String openid) {
        return getKey(platformId, appid, openid, KEY_PREFIX_ORDER_NOPAY);
    }

    public String getKeyPrefixMpNosign(Integer platformId, String appid, String openid) {
        return getKey(platformId, appid, openid, KEY_PREFIX_MP_NOSIGN);
    }


}

