package com.book.push.controller;

/**
 * created in 2021/8/17
 * Project: book-store
 *
 * @author win7
 */

import com.book.dao.cps.pojo.AdminConfig;
import com.book.dao.cps.pojo.Ophost;
import com.book.dao.cps.pojo.User;
import com.book.push.cons.UrlCons;
import com.book.push.service.dao.*;
import com.book.push.service.redis.RedisService;
import com.book.push.service.wx.WxThirdPartService;
import com.book.push.utils.Md5Util;
import com.book.push.utils.UrlUtil;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.api.WxConsts;
import me.chanjar.weixin.common.bean.WxOAuth2UserInfo;
import me.chanjar.weixin.common.bean.oauth2.WxOAuth2AccessToken;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.common.service.WxOAuth2Service;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletRequest;

/**
 *
 * @author
 */
@Slf4j
@Controller
@RequestMapping("/login")
public class LoginController {
    @Autowired
    private AdminConfigService adminConfigService;
    @Autowired
    private WxThirdPartService thirdPartService;

    @Value("${book.debug}")
    private Boolean debug;
    @Autowired
    private RedisService redisService;
    @Autowired
    private OpHostService opHostService;
    @Autowired
    private UserService userService;
    @Autowired
    private UserSilentService userSilentService;
    @Autowired
    private ConfigService configService;

    /**
     * //FIXME
     * <p>
     * 重定向urL，需要改
     */


    @GetMapping
    public String userLogin(HttpServletRequest request, @RequestParam(required=false) String appid, @RequestParam(required=false) String code, @RequestParam(required=false) String state) {


        WxOAuth2AccessToken oAuth2rRefreshToken = null;
        String domainName = request.getServerName();
        if (code == null) {

            return fromFront(request, domainName);

        } else {
            AdminConfig adminConfig = adminConfigService.selectByAppid(appid);
            oAuth2rRefreshToken = thirdPartService.getOAuth2AccessTokenByCode(adminConfig.getPlatformId(), appid, code);
            log.info(oAuth2rRefreshToken.toString());
            User user = null;
            try {
                WxOAuth2Service oAuth2Service = thirdPartService.getWxOpenServiceByAuthorizerAppid(adminConfig.getAppid())
                        .getWxOpenComponentService().getWxMpServiceByAppid(adminConfig.getAppid()).getOAuth2Service();
                user = userService.selectByOpenId(oAuth2rRefreshToken.getOpenId());
                if (user==null){
                    WxOAuth2UserInfo userInfo = oAuth2Service.getUserInfo(oAuth2rRefreshToken, "");
                    log.info(userInfo.toString());
                    user = userService.createUserByWxOAuth2UserInfo(userInfo,adminConfig.getAdminId());

                }
            } catch (WxErrorException e) {
                e.printStackTrace();
                user = userService.createUserByOpenid(oAuth2rRefreshToken.getOpenId(), adminConfig.getAdminId());
            }
            return buildRedirectUrl(getFontUlr(redisService.get(state),adminConfig,user.getId()));
        }




    }

    private String fromFront(HttpServletRequest request, String domainName) {
        WxOAuth2AccessToken oAuth2rRefreshToken=null;
        String referer = request.getHeader("referer");
        if (referer == null) {
            return buildRedirectUrl(null);
        }
        String state = Md5Util.md5(referer);



        String appid = getAppIdByReferer(referer);
        AdminConfig adminConfig = adminConfigService.selectByAppid(appid);
//        oAuth2rRefreshToken = thirdPartService.getOAuth2AccessTokenByRefreshToken(adminConfig.getPlatformId(), adminConfig.getAppid());
        WxOAuth2Service oAuth2Service = thirdPartService.getWxOpenServiceByAuthorizerAppid(adminConfig.getAppid())
                .getWxOpenComponentService().getWxMpServiceByAppid(adminConfig.getAppid()).getOAuth2Service();
        if (oAuth2rRefreshToken == null) {
            if (redisService.get(state) == null) {
                redisService.set(state, referer);
            }
            String redirectUrl = String.format(UrlCons.URL_OAUTH2_REDIRECT_LOGIN, domainName);

            String auto2Url = oAuth2Service.buildAuthorizationUrl(redirectUrl, WxConsts.OAuth2Scope.SNSAPI_USERINFO, state);
            return buildRedirectUrl(auto2Url);
        } else {

            //todo 废弃
            User user = null;
            try {
                log.info(oAuth2rRefreshToken.toString());
                user = userService.selectByOpenId(oAuth2rRefreshToken.getOpenId());
                if (user==null){
                    WxOAuth2UserInfo userInfo = oAuth2Service.getUserInfo(oAuth2rRefreshToken, "");
                    log.info(userInfo.toString());
                    user = userService.createUserByWxOAuth2UserInfo(userInfo,adminConfig.getAdminId());

                }

            } catch (WxErrorException e) {
                e.printStackTrace();
                user = userService.createUserByOpenid(oAuth2rRefreshToken.getOpenId(), adminConfig.getAdminId());
            }
            return buildRedirectUrl(getFontUlr(referer,adminConfig,user.getId()));
        }
    }


    private String getAppIdByReferer(String referer) {
        if (referer == null) {
            return null;
        }
        String domainName = UrlUtil.getDomainName(referer);
        return domainName.split("\\.")[0];

    }




    private String buildRedirectUrl(String fontUlr) {
        if (fontUlr == null || !fontUlr.toLowerCase().startsWith("http")) {

            String msg = (fontUlr == null ? "登陆失败，请在网页内登录" : fontUlr);
            return msg;
        }
        log.info("跳转：" +fontUlr);
        return "redirect:" + fontUlr;
    }

    private String getFontUlr(String referer, AdminConfig adminConfig, Long userId) {
        if (referer == null) {
            return "登录失败啦，请在网站里面进行登录";
        }
        Integer ophostId = adminConfig.getOphostId();
        Ophost ophost = opHostService.selectById(ophostId);
//        String s = redisService.get(referer);
        String url = UrlUtil.replaceDomain(referer, adminConfig.getAppid() + "." + ophost.getHost());
        return UrlUtil.addParams(url, String.format(UrlCons.URL_PARAMS, adminConfig.getAdminId(), userId));
    }



}
