package com.book.push.task;

import com.book.dao.cps.pojo.AdminConfig;
import com.book.dao.cps.pojo.Custom;
import com.book.dao.cps.pojo.User;
import com.book.dao.utils.DateUtils;
import com.book.push.service.dao.AdminConfigService;
import com.book.push.service.dao.CustomService;
import com.book.push.service.dao.UserService;
import com.book.push.service.push.PushService;
import com.book.push.service.push.vo.CustomMessage;
import com.book.push.service.wx.WxThirdPartService;
import com.book.push.utils.JsonUtils;
import me.chanjar.weixin.mp.bean.kefu.WxMpKefuMessage;
import me.chanjar.weixin.mp.builder.kefu.NewsBuilder;
import me.chanjar.weixin.open.api.WxOpenMpService;
import me.chanjar.weixin.open.api.WxOpenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * created in 2021/8/17
 * Project: book-store
 * 客服消息
 *
 * @author win7
 */

@Component
public class KefuTask {
    @Autowired
    private CustomService customService;
    @Autowired
    private AdminConfigService adminConfigService;
    @Autowired
    private WxThirdPartService wxThirdPartService;
    @Autowired
    private UserService userService;
    @Autowired
    private PushService pushService;
    @Scheduled(fixedRate = 1000 * 60 * 10)
    public void run() {
        Integer now = DateUtils.getNow();
        Integer to = now + (1000 * 60 * 10);
        List<Custom> customs = customService.selectListBetweenSendTime(now, to);
        for (Custom custom : customs) {
            Integer sendtime = custom.getSendtime();
            Integer delay = sendtime - DateUtils.getNow();
            Integer adminId = custom.getAdminId();
            AdminConfig adminConfig = adminConfigService.selectByAdminId(adminId);
            String appid = adminConfig.getAppid();
            Integer platformId = adminConfig.getPlatformId();
            String userJsonStr = custom.getUserJson().toString();
            userJsonStr = userJsonStr.substring(1,userJsonStr.length()-1).replace("\\","");
            Custom.UserJson userJson = JsonUtils.getObject(userJsonStr, Custom.UserJson.class);
            List<User> userList = getListByjson(userJson, adminId);
            String messageJson = custom.getMessageJson().toString().replace("\\","");
            messageJson = messageJson.replace("\\","").substring(1,messageJson.length()-1);

            List<Custom.MessageJson> list = JsonUtils.getList(messageJson, Custom.MessageJson.class);

            List<WxMpKefuMessage> messageList = new ArrayList<>();
            for (User user:userList){
                NewsBuilder newsBuilder = new NewsBuilder().toUser(user.getOpenid());
                for (Custom.MessageJson  message:list){
                    WxMpKefuMessage.WxArticle wxArticle = new WxMpKefuMessage.WxArticle();
                    wxArticle.setTitle(message.getTitle());
                    wxArticle.setPicUrl(message.getImage());
                    wxArticle.setUrl(message.getUrl());
                    newsBuilder.addArticle(wxArticle);

                }
                WxMpKefuMessage kefuMessage = newsBuilder.build();
                messageList.add(kefuMessage);
            }
            custom.setSendNum(messageList.size());
            WxOpenService wxOpenService = wxThirdPartService.getWxOpenServiceByPlatFormId(platformId);
            WxOpenMpService wxMpService = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid);
            CustomMessage customMessage = new CustomMessage();
            customMessage.setCustom(custom);
            customMessage.setMessageList(messageList);
            customMessage.setCustomService(customService);

            pushService.addCustomMessagePushTask(wxMpService, customMessage, delay, TimeUnit.SECONDS);


        }
    }

    private List<User> getListByjson(Custom.UserJson userJson, Integer channelId) {
        // "{\"sex\":-1,\"tag\":-1,\"consume\":-1,\"kandian\":-1,\"subscribe_time\":-1,\"all\":\"0\"}"
//        "{\"sex\":2,\"mobile_system\":2,\"tag\":8,\"consume\":4,\"kandian\":2,\"subscribe_time\":7,\"all\":\"0\"}"

        Map map = new HashMap<>();
        map.put("channel_id", channelId);
        if (!"1".equals(userJson.getAll())) {
            if (userJson.getKandian() == 1) {
                map.put("kandian_max", "500");
            } else if (userJson.getKandian() == 2) {
                map.put("kandian_max", "2000");
            } else if (userJson.getKandian() == 3) {
                map.put("kandian_min", "2000");
            }

            if (userJson.getSex() != -1) {
                map.put("sex", userJson.getSex());
            }
            if (userJson.getConsume() == 1) {
                map.put("is_pay", "0");
            } else if (userJson.getConsume() == 4) {
                map.put("is_pay", "1");
            }
            if (userJson.getTag() != -1) {
                map.put("tag", userJson.getTag());
            }
            if (userJson.getSubscribe_time() != -1) {
                int subscribeTime = -1;
                if (userJson.getSubscribe_time() == 6) {
                    subscribeTime = DateUtils.getNow() - 12 * 60 * 60;
                } else if (userJson.getSubscribe_time() == 7) {
                    subscribeTime = DateUtils.getNow() - 24 * 60 * 60;
                } else if (userJson.getSubscribe_time() == 8) {
                    subscribeTime = DateUtils.getNow() - 48 * 60 * 60;
                } else if (userJson.getSubscribe_time() == 9) {
                    subscribeTime = DateUtils.getNow() - 7 * 24 * 60 * 60;
                } else if (userJson.getSubscribe_time() == 10) {
                    subscribeTime = DateUtils.getNow() - 15 * 24 * 60 * 60;
                }
                map.put("subscribe_time", subscribeTime);
            }

        }
        List<User> users = userService.selectListByPushMap(map);
        return users;

    }


}
